pack:
  name: {{ pack_name }}
  namespace: {{ namespace }}
  version: 0.0.0
  description: {{ description }}
  visibility: private
  author: {{ author }}

commands:
  - name: hello
    description: A simple hello world command
    enabled: true
