{
  "name": "{{ dashboard_name }}",
  "version": "0.0.0",
  "type": "module",
  "scripts": {
    "dev": "vite",
    "build": "vite build",
    "preview": "vite preview"
  },
  "dependencies": {
    "react": "^19.2",
    "react-dom": "^19.2"
  },
  "devDependencies": {
    "@vitejs/plugin-react": "^4.0",
    "vite": "^5.0",
    "typescript": "^5.3"
  }
}
