[project]
name = "{{ pack_name }}"
version = "0.0.0"
description = "{{ description }}"
requires-python = ">=3.11"
dependencies = ["huitzo-sdk>=0.0.1"]
authors = [{ name = "{{ author }}", email = "{{ author_email }}" }]

[project.optional-dependencies]
dev = ["pytest>=9.0", "pytest-asyncio>=1.0", "pytest-cov>=7.0", "mypy>=1.19", "ruff>=0.14"]

[project.entry-points."huitzo.commands"]
"@{{ namespace }}/{{ pack_name }}/hello" = "{{ module_name }}.commands.hello:hello_world"

[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[tool.hatch.build.targets.wheel]
sources = ["src"]

[tool.pytest.ini_options]
asyncio_mode = "auto"
testpaths = ["tests"]

[tool.mypy]
strict = true

[tool.ruff]
target-version = "py311"
line-length = 100
