# {{ pack_name }}

{{ description }}

## Installation

```bash
pip install -e .
```

## Usage

```bash
# Run tests
pytest tests/ -v

# Validate
huitzo pack validate

# Build
huitzo pack build

# Develop
huitzo pack dev
```

## Development

This pack includes a simple `hello` command that demonstrates the Huitzo SDK pattern.

### Running Tests

```bash
pytest tests/ -v --cov=src/{{ module_name }}
```

### Type Checking

```bash
mypy src/ --strict
```

### Formatting

```bash
ruff format .
ruff check .
```

## License

Proprietary - Huitzo Inc.
