"""
Module: {{ command_func }}
Description: {{ description }}

Implements:
    - docs/sdk/commands.md
"""
{% if use_pydantic %}
from pydantic import BaseModel, Field
{% endif %}
from huitzo_sdk import Context, command

{% if use_pydantic %}

class {{ args_class }}(BaseModel):
    """Arguments for the {{ command_name }} command."""

    # TODO: Define your command arguments here
    name: str = Field(default="World", description="Example argument")

{% endif %}

@command("{{ command_name }}", namespace="{{ namespace }}"{% if timeout != 60 %}, timeout={{ timeout }}{% endif %})
async def {{ command_func }}(args: {% if use_pydantic %}{{ args_class }}{% else %}dict{% endif %}, ctx: Context) -> dict:
    """{{ description }}

    Args:
        args: Command arguments
        ctx: Huitzo context

    Returns:
        Command result
    """
    # TODO: Implement your command logic here
    return {"status": "ok"}
