"""
Module: test_{{ command_func }}
Description: Tests for {{ command_name }} command
"""

import pytest

from {{ module_name }}.commands.{{ command_file }} import {{ command_func }}{% if use_pydantic %}, {{ args_class }}{% endif %}


@pytest.mark.asyncio
async def test_{{ command_func }}_returns_result() -> None:
    """Test {{ command_func }} returns expected result."""
    result = await {{ command_func }}({% if use_pydantic %}{{ args_class }}(){% else %}{}{% endif %}, None)  # type: ignore
    assert isinstance(result, dict)
    assert "status" in result
