import { useState } from 'react'
import './App.css'

export default function App() {
  const [count, setCount] = useState(0)

  return (
    <div className="container">
      <h1>{{ dashboard_name }}</h1>
      <p>A Huitzo Dashboard</p>

      <button onClick={() => setCount(count + 1)}>
        Count: {count}
      </button>

      <p>
        Edit <code>src/App.tsx</code> to start building your dashboard.
      </p>
    </div>
  )
}
